import { Component, OnInit, ViewChild } from '@angular/core';
import { IgxGridComponent } from 'igniteui-angular';

@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.scss']
})
export class AppComponent {
  title = 'igx-grid-drag-and-drop';
  data: any[];
  columns: any[];
  @ViewChild('grid1', { read: IgxGridComponent, static: true }) grid1: IgxGridComponent;

  constructor() {
    // 列の定義
    this.columns = [
      { field: 'ID', headerText: 'ID', width: 100, resizable: false, pinned: true },
      { field: 'Name', headerText: '名前', width: 150, resizable: false, pinned: true },
      { field: 'Location', headerText: '勤務地', width: 100, resizable: true },
      { field: 'Department', headerText: '部署', width: 100, resizable: true },
      { field: 'PostalCode', headerText: '郵便番号', width: 120, resizable: true },
      { field: 'Prefecture', headerText: '住所１', width: 100, resizable: true },
      { field: 'Address', headerText: '住所２', width: 250, resizable: true },
      { field: 'Phone', headerText: '電話番号', width: 150, resizable: true }
    ];
    // データ
    this.data = [
      { 'ID': 105, 'Name': '森上 偉久馬', 'Location': '東京本社', 'Department': '第一営業', 'PostalCode': '1900003', 'Prefecture': '東京都', 'Address': '立川市栄町 4-18-XX', 'Phone': '(0425)25-05XX' },
      { 'ID': 107, 'Name': '葛城 孝史', 'Location': '東京本社', 'Department': '第二営業', 'PostalCode': '1530064', 'Prefecture': '東京都', 'Address': '目黒区下目黒 3-16-XX', 'Phone': '(03)3714-75XX' },
      { 'ID': 110, 'Name': '加藤 泰江', 'Location': '東京本社', 'Department': '第一営業', 'PostalCode': '1520021', 'Prefecture': '東京都', 'Address': '目黒区東ケ丘 1-5-X', 'Phone': '(03)3421-16XX' },
      { 'ID': 204, 'Name': '川村 匡', 'Location': '大阪支社', 'Department': '営業開発', 'PostalCode': '5610857', 'Prefecture': '大阪府', 'Address': '豊中市服部寿町 1-11-5-XXX', 'Phone': '(06)6862-91XX' },
      { 'ID': 207, 'Name': '松沢 誠一', 'Location': '大阪支社', 'Department': '営業開発', 'PostalCode': '5690000', 'Prefecture': '大阪府', 'Address': '高槻市桂本 6-28-X', 'Phone': '(0726)77-32XX' },
      { 'ID': 210, 'Name': '成宮 真紀', 'Location': '大阪支社', 'Department': '営業一', 'PostalCode': '5630033', 'Prefecture': '大阪府', 'Address': '池田市住吉 2-11-XX', 'Phone': '(0727)62-27XX' },
      { 'ID': 304, 'Name': '山本 雅治', 'Location': '北九州支社', 'Department': '営業一', 'PostalCode': '8160088', 'Prefecture': '福岡県', 'Address': '福岡市博多区板付 7-6-XX', 'Phone': '(092)581-78XX' },
      { 'ID': 305, 'Name': '青木 俊之', 'Location': '北九州支社', 'Department': '営業一', 'PostalCode': '8112205', 'Prefecture': '福岡県', 'Address': '粕屋郡志免町大字別府 844-XX', 'Phone': '(092)936-45XX' },
      { 'ID': 307, 'Name': '小川 さよ子', 'Location': '北九州支社', 'Department': '営業二', 'PostalCode': '8160825', 'Prefecture': '福岡県', 'Address': '春日市伯玄町 2-X', 'Phone': '(092)591-64XX' },
    ];
  }

  onDropAllowed(args) {
    args.cancel = true;
    const event = args.originalEvent;
    const currRowIndex = this.getCurrentRowIndex(this.grid1.rowList.toArray(),
      { x: event.pageX, y: event.pageY });
    // ドラッグで移動した行をグリッドから削除
    this.grid1.deleteRow(args.dragData.rowID);
    // ドラッグで移動した行を、新しいインデックスに挿入
    this.data.splice(currRowIndex, 0, args.dragData.rowData);
  }

  private getCurrentRowIndex(rowList, cursorPosition) {
    for (const row of rowList) {
      const rowRect = row.nativeElement.getBoundingClientRect();
      if (cursorPosition.y > rowRect.top && cursorPosition.y < rowRect.bottom &&
        cursorPosition.x > rowRect.left && cursorPosition.x < rowRect.right) {
        // ドロップ位置の行インデックスをリターン
        return this.data.indexOf(this.data.find((r) => r.ID === row.rowData.ID));
      }
    }
    throw new Error('行インデックスの取得に失敗しました。');
  }
}
